/*
    This file is part of COMMON AVR UTILS.

    COMMON AVR UTILS is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License (LGPL)
    as published by the Free Software Foundation, either version 3 of
    the License, or (at your option) any later version.

    COMMON AVR UTILS is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License at <http://www.gnu.org/licenses/>
    for more details.

    (C) Georg-Johann Lay <georgjohann@web.de>
*/

#include "taster.h"

volatile uint8_t taster;

void get_taster (taste_t * const ptast, uint8_t tast)
{
  // Nichts tun, falls die Anwendung den Wert aus `taster' noch nicht
  // abgehohlt hatden
  if (taster)
    return;

  // ist Taster gedrückt? (low aktiv)
  tast = ! tast;

  // alten Taster-Zustand lesen
  uint8_t taster_old = ptast->old;
  uint8_t mode  = ptast->mode;

  // skip?
  if (KM_SKIP == mode)
    tast = taster_old;

  // Taster-Zustand merken
  ptast->old = tast;

  // Was wurde gedrückt/losgelassen...?
  uint8_t pressed =  taster_old &  tast;
  uint8_t release =  taster_old & ~tast;
  uint8_t press   = ~taster_old &  tast;

  tast = 0;

  uint8_t delay = ptast->delay;
  uint8_t key   = ptast->key;

#ifdef TASTER_SAME_MODE
  // Let GCC throw away superflous code if there is some extra
  // knowledge. Even though there is more C code in that case
  // the code size will reduce. Confused?
  mode = TASTER_SAME_MODE;
#endif // TASTER_SAME_MODE    

  // Taste wurde eben gedrückt
  if (press)
  {
    if (mode != KM_LONG)
      tast = key;

    delay = 0;
  }
  // Taste bleibt gedrückt
  else if (pressed)
  {
    if (delay < 0xfe)
      delay++;
  }
  // Taste wurde losgelassen
  else if (release)
  {
    if (mode == KM_LONG && delay != 0xff)
      tast = key;
  }

  // Langer Tastendruck?
  if (mode == KM_LONG)
  {
    if (delay == KEYDELAY_LONG)
    {
      tast = KEY_LONG + key;
      delay = 0xff;
    }
    ptast->delay = delay;
  }
  // Auto-Repeate
  else if (mode == KM_REPEAT)
  {
    if (delay == KEYREPEAT_DELAY)
    {
      tast = key;
      delay = KEYREPEAT_DELAY - KEYREPEAT;
    }
    ptast->delay = delay;
  }

  taster = tast;
}
